import csv
from django.http import HttpResponse

#This class is responsible for creating the csv file from the model data and downloading it.
class CSV_Downloader(object):
    #constructor to initialize,  
    #data:  it is model data which forms the content of the csv file.
    #content type:  initialize the content type of csv file
    #content_disposition: initialize the content_disposition of the download csv file
    def __init__(self, data):
        self.data=data
        self.content_type='text/csv'
        self.content_disposition = 'attachment;filename=export.csv'
        
    #method downloads the csv file using the model data as content.
    def download_csv_file(self):
        metadata=self.data.model._meta
        model =self.data.model
        csv_file_response = HttpResponse(content_type=self.content_type)
        # force download.
        csv_file_response['Content-Disposition'] = self.content_disposition
        #csv writer to create the response
        csv_writer=csv.writer(csv_file_response)
        name_of_fields=[field.name for field in metadata.fields]
        #header information
        csv_writer.writerow(name_of_fields)
        #writes the row content of csv file
        for unit in self.data:
            csv_writer.writerow([getattr(unit, field) for field in name_of_fields])
        print(csv_file_response)
        return csv_file_response


from .download_csv import CSV_Downloader
from .fetch_best_books import BooksRecommender

#DownloadCSVAndRecommendBooks handles two core functionality of library:
#recommend books based on a book name.
#convert model data to csv and download it.
class DownloadCSVAndRecommendBooks(object):
    #constructor to initialize fields for recommendation of book
    def __init__(self, join_on, book_title_column, user_column, value_column):
        self.joinon=join_on
        self.book_col_name=book_title_column
        self.user_col_name=user_column
        self.value_col=value_column
        
    def recommended_books(self,bookName, issued_book_df, book_df):
        
        book_recc=BooksRecommender(self.joinon, self.book_col_name, self.user_col_name,self.value_col)
        response=book_recc.get_books(bookName, issued_book_df, book_df)
        return response
        
    @classmethod
    def download_csv(model_data):
            csv_downloader=CSV_Downloader(model_data)
            downloaded_csv=csv_downloader.download_csv_file()
            return downloaded_csv
            
            


import pandas as pd
import psycopg2 as pg
import pandas as pds
from sqlalchemy import create_engine
from sqlalchemy import create_engine
from scipy.sparse import csr_matrix
from sklearn.neighbors import NearestNeighbors

#this class recommends similar behaviour based on the popularity of the book amoung the library users. Popularity is decided based on 
#how many users have issued any particular book. And studies the similarity between types of book opted by the users. 
class BooksRecommender(object):
    #constructor to initialize 
    # join_on: common column to join the datasets
    # book_title_column: book title column name in the dataset
    # user_column: user column name in the dataset
    # value_column: value column name, based on which analysis to be performed.
    def __init__(self, join_on, book_title_column, user_column, value_column):
        self.join_on=join_on
        self.book_column=book_title_column
        self.user_col=user_column
        self.val_col=value_column
        
    # method helps in getting recommended similar books
    #book_name: name of the book, based on which analysis to be performed to get the recommended books.
    # issued_books: dataframe to holds the details of issued books to the users in the library.
    #books: dataframe to hold the details of all the books in the library database.
    #returns the recommended books for book_name value.
    def get_books(self, book_name, issued_books, books):
        try:
            issued_books_with_book_information = issued_books.merge(books, on=self.join_on)
            recommended_books_name=[]
            #create a pivot with column values as user_id and index as book_name and value will be the how many books issued to the user
            #basically books which are read by user are taken similarity.
            user_book_pivot = issued_books_with_book_information.pivot_table(columns=self.user_col, index=self.book_column, values=self.val_col)
            user_book_pivot.fillna(0, inplace=True)
            user_book_sparse = csr_matrix(user_book_pivot)
            #brute algorithen helps in measuring the distance of every point with other point
            model = NearestNeighbors(algorithm='brute')
            model.fit(user_book_sparse)
            indx=user_book_pivot.index.values.tolist().index(book_name)
            distances, suggestions = model.kneighbors(user_book_pivot.iloc[indx,: ].values.reshape(1, -1))
            for i in range(len(suggestions)):
                recommended_books_name.append(user_book_pivot.index[suggestions[i]])
            return recommended_books_name
        except:
            recommended_books_name=[]
            return recommended_books_name
            


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    
    long_description = fh.read()
    
    setuptools.setup(
    
    name="csv_converter_and_book_recommender_pkg-Preeti_Rawat",
    # Replace with your own username above
    version="0.0.2",
    author="Preeti_Rawat",
    author_email="x20233507@student.ncirl.ie",
    description="A model to csv converter and book recommender package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/bookrecommenderandcsvconverter",
    packages=setuptools.find_packages(),
    # if you have libraries that your module/package/library
    #you would include them in the install_requires argument
    install_requires=[''],
    classifiers=[
        
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
